/*
 * Copyright (C) 1999 Red Hat Software, Inc.
 *	Michael Fulbright <drmike@redhat.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gnome.h>

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>


#include "wmconfig.h"

#define TERMINAL_USED "xterm -e"

/*
 * This reads & parses a system configuration file
 */
int read_gnome_file(const char *filename, const char *group)
{
    GnomeDesktopEntry *dentry;
    char               *package, *name, *exec;
    int                i, add_for_terminal;

/*    fprintf(stderr, "scanning file %s in group %s\n",filename, group); */
    
    if (!filename)
	return -1;

    dentry = gnome_desktop_entry_load(filename);

    /* skip if it has a tryexec */
    if (!dentry || dentry->tryexec) {
	gnome_desktop_entry_free(dentry);
	return 0;
    }

    /* convert to a wmconfig style entry
     *
     *  package is same as GNOME exec field
     *
     *     wmconfig            GNOME entry
     *     --------            -----------
     *      name                  name
     *      description           comment
     *      group                 group arg
     *      exec                  exec
     */

    if (!dentry->name && !dentry->exec[0]) {
	gnome_desktop_entry_free(dentry);
	return -1;
    }

    if (dentry->name) {
	package = dentry->name;
    } else if (dentry->exec[0]) {
	package = dentry->exec[0];
    }

    name = package;
    
    exec = strdup("");
    add_for_terminal = 0;
    for (i=0; i < dentry->exec_length; i++) {
	if (!dentry->exec[i])
	    break;

	if (! add_for_terminal && dentry->terminal) {
		add_for_terminal = strlen(TERMINAL_USED);
		exec = realloc(exec, strlen(exec)+add_for_terminal+2);
		strcat(exec, TERMINAL_USED);
		strcat(exec, " ");
	}

	exec = realloc(exec, strlen(exec)+strlen(dentry->exec[i])+2);
	strcat(exec, dentry->exec[i]);
	strcat(exec, " ");
    }

    exec = realloc(exec, strlen(exec)+2);
    strcat(exec, "&");
    update_package(package, "exec", exec);
    update_package(package, "name", name);
    update_package(package, "group", group);
    
    if (dentry->comment)
	update_package(package, "description", dentry->comment);

    free(exec);
    gnome_desktop_entry_free(dentry);
    return 0;
}

/*
 * This function parses the system configuration from
 * a confdir and fills in the structures needed by
 * this program
 */
int parse_gnome_files(const char *dir_from, char *start_group)
{
    struct stat st;
    int		ret;
    DIR		*sys_dir;
    struct dirent **namelist;
    int		nr_files;
    int 	i;
    const char  *curgroup;

    if (dir_from == NULL)
	return PARSE_OK;

    curgroup = (start_group) ? start_group : "";
    
    ret = stat(dir_from, &st);
    if (ret != 0) {
	fprintf(stderr, "%s: %s\n",
		__FUNCTION__, sys_errlist[errno]);
	return PARSE_ABORT;
    }
    /* okay, it exist... do some more tests */
    if (!S_ISDIR(st.st_mode)) {
	fprintf(stderr, "%s: %s is not a directory\n",
		__FUNCTION__, dir_from);
	return PARSE_ABORT;
    };
    /* ...and is a directory */
    sys_dir = opendir(dir_from);
    if (sys_dir == (DIR *)NULL) {
	fprintf(stderr, "%s: can not read system config files from %s\n",
		__FUNCTION__, dir_from);
	return PARSE_ABORT;
    };
    closedir(sys_dir);
    
    /* read the whole directory in one pass */
    nr_files = scandir(dir_from, &namelist, NULL, alphasort);
    for (i = 0; i < nr_files; i++) {
	if ( (strcmp(namelist[i]->d_name, ".") != 0) &&
	     (strcmp(namelist[i]->d_name, "..") != 0) ) {
	    char filename[PATH_MAX];
	    int ret_val;

#ifndef WANT_DOT_FILES
	    if (namelist[i]->d_name[0] == '.')
		continue;
#endif	    
	    snprintf(filename, PATH_MAX, "%s/%s", dir_from, namelist[i]->d_name);

	    /* is this a directory to recurse into? */
	    ret = stat(filename, &st);
	    if (ret)
		return PARSE_ABORT;

	    if (S_ISDIR(st.st_mode) &&
		(strcmp(namelist[i]->d_name, "Settings") != 0)) {
		char *newgroup;
		int  len;

		len = strlen(curgroup) + strlen(namelist[i]->d_name) + 2;
		newgroup = malloc(len);
		if (*curgroup)
		    snprintf(newgroup,len-1,"%s/%s",curgroup,
			     namelist[i]->d_name);
		else
		    strcpy(newgroup, namelist[i]->d_name);

		ret_val = parse_gnome_files(filename, newgroup);
		free(newgroup);
		if (ret_val != 0) {
		    fprintf(stderr, "\tReading direcory \"%s\" failed !\n",
			    filename);
		}
	    } else {
		ret_val = read_gnome_file(filename, curgroup);
		if (ret_val != 0) {
		    fprintf(stderr, "\tReading file \"%s\" failed !\n",
			    filename);
		}
	    }
	}
	free(namelist[i]);	
    }
    if (nr_files && namelist)
	free(namelist);
/*     print_groups(root_group, 0); */
    
    return PARSE_OK;
}


